<?php
/**
 * Contains All methods related to doing http work in this plugin
 *
 * General HTTP
 * @package WP-Favicons
 * @since 0.4.0
 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
 * @copyright GPL 2
 */

namespace leau\co\wp_favicons;

//
// see: http://wp.leau.co/2011/03/18/handling-redirects-for-wp-favicons-with-redirects-set-to-0-with-wordpress-wp_remote_get/
// for more info an open cases on some URI's
//

if (!class_exists("Http"))
{

	/**
	 * Contains All methods doing http work and related utils
	 * The core Method is " GetIconFromUri" which tries to get an icon from
	 * a certain URI.
	 *
	 * @since 0.1
	 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 * @copyright GPL 2
	 * @static class containing http functions
	 */
	class Http
	{
		const FLAG_NOK = 'FLAG_NOK';
		/**
		 * Enter description here ...
		 * @var unknown_type
		 */
		const FLAG_TOO_MANY_COUNTS = 'TOO_MANY_COUNTS';
		/**
		 * Enter description here ...
		 * @var unknown_type
		 */
		const FLAG_DATA_FOUND = 'DATA_FOUND';
		/**
		 * Enter description here ...
		 * @var unknown_type
		 */
		const FLAG_ICON = 'ICON';
		const FLAG_PAGE = 'PAGE';
		const MAX_REDIRECTS = 6;

		/**
		 * @var singleton instance
		 */
		private static $instance;

		/** private constructor function
		 *  to prevent external instantiation
		 */
		private function __construct()
		{
		}

        /**
		 * clone method
		 */
		private function __clone()
		{
		}

		/**
		 * getInstance method
		 */
		public static function getInstance()
		{
			if(!self::$instance)
			{
				self::$instance = new self();
			}
			return self::$instance;
		}

		/* ------------------------------------------------------------ */

		/**
		 * Returns max redirection count
		 * Important that this is 0 we do our redirection ourself
		 * Has to be public to act in callback function
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 	 * @copyright GPL 2
	 	 * @return int maximum amount of redirections
		 */
		public function SetDirectionCount()
		{
			return 0;
		}

		/**
		 * Returns the default useragent
		 * Has to be public to act in callback function
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 	 * @copyright GPL 2
	 	 * @return string User Agent String
		 */
		public function SetHeaderUserAgent()
		{
			$version = Config::GetPluginVersion();
			return "WordPress/WP-Favicons/$version; " . get_bloginfo( 'url' );
		}

    	/* ------------------------------------------------------------ */

		/**
		 * Parses an embedded icon or icon href from a rel href string
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 	 * @copyright GPL 2
	 	 * @param object $xpath xpath object needed to find base href if needed
		 * @param string $XpathExpr expression used merely for statistics logging
		 * @param string $hrefstring the string we are parsing
		 * @param string $uri the original uri request for logging
		 * @returns array $favicon_array as used all over the place
		 */
		function GetIconFromHrefString( $xpath, $XpathExpr, $hrefstring, $favicon)
		{
			$instance = self::getInstance();

			/* it can contain 2 things either an embedded base64 encoded string
			 * or a href */

			/* option 1: embedded */
			if (strtolower(substr(trim($hrefstring),0,4))=='data')
			{
				Log::F($favicon,'[PAGE] embedded image found');

				/* 1 explode the parts of the href */

				list($type,$rest) = explode(":", $hrefstring);
				list($mimetype,$rest) = explode(";",$rest);
				list($encoding,$imagedata) = explode(",", $rest);
				$type = trim($type);
				$mimetype = trim($mimetype);
				$encoding = trim($encoding);
				$imagedata = trim($imagedata);

				/* 2 check if base 64 */

				if (!$encoding=='base64') { return false; }

				/* 3 verify and store (we get filters and type) */
				// if ($return_array = filters_verify_image::VerifyImage($return_array))
				// takes place in page class

				$favicon->SetFaviconUri($favicon->GetUri(). "[" . $imagedata . "]");
				$favicon->SetSource('sources_page' .
						':' . $XpathExpr . ':' . 'embedded');
				$favicon->SetData(base64_decode($imagedata));
				$favicon->AddUriToUriArr($favicon->GetFaviconUri());
				return $instance::FLAG_DATA_FOUND;
			}
			/* option 2: not embedded */
			else
			{
				Log::F($favicon,'[PAGE] href found');
				$favicon->SetSource('sources_page'
				.':' . $XpathExpr . ':' . 'href');
				return $instance->CallAgain($favicon,
					Http::getFullFaviconPath($favicon->LastUriAddedToUriArr(), $hrefstring, $xpath));
			}
		}

		/* ------------------------------------------------------------ */

		/**
		 * gets a REL ICON href string from a HTML page
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 	 * @copyright GPL 2
	 	 * @param string $data the HTML page in a string
	 	 * @param string $uri the uri we are requesting
		 */
		function GetHrefStringFromHTMLPage($favicon)
		{
			/*
			 * these could be merged for speed but we use them for statistics on usage
			 * see: http://bit.ly/hBdhJV kudos http://stackoverflow.com/users/459897/dr-molle
			 */
			$instance = self::getInstance();
			Log::F($favicon,'FINDICON process (in page)');
			/* quick shortcut */
			if (!( strpos("-".strtolower($favicon->GetData()),strtolower('<head')) &&
				   strpos("-".strtolower($favicon->GetData()),strtolower('<body'))
				 )
			   )
			{
				//
				// @todo some sites do a meta http-equiv = "refresh" and end up in
				// this one we should use the "url" in those meta tags also for
				// getting the full page path. Since they will always end up
				// in this place...but a meta refresh could lead to a lot of
				// stuff... so it is equal to a 302/301 ...
				//
				Log::F($favicon,'icon with wrong mime (probably)');
				return $instance::FLAG_DATA_FOUND;
			}

			$_xPathExpressions = array(
				"//head/link[@rel='shortcut icon']",
				"//head/link[@rel='icon']",
				"//link[@rel='shortcut icon']",
				"//link[@rel='icon']",
				"//link[translate(@rel,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz') = 'shortcut icon']",
				"//link[translate(@rel,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz') = 'icon']",
			);
			$dom = new \DOMDocument();
			libxml_use_internal_errors(true);
			$dom->strictErrorChecking = false;

			if (!$dom->loadHTML($favicon->GetData()))
			{
				foreach (libxml_get_errors() as $error)
				{
				}
				libxml_clear_errors();
			}
			else
			{
				$xpath = new \DOMXPath($dom);
				foreach($_xPathExpressions as $xPathExpression)
				{
					if ($hrefstring=$instance->XpathHrefCheck($xpath, $xPathExpression))
					{
						Log::F($favicon,'image rel tag found in page : ' . $hrefstring );

						return $instance->GetIconFromHrefString($xpath, $xPathExpression,
							$hrefstring, $favicon);
					}
					else
					{
						Log::F($favicon,'image rel tag NOT found in page with : ' . $xPathExpression);
					}
				}
				/* if not checked before try /favicon.ico in the root */
				// @todo: before testing /favicon.ico in the root do a check
				// for the root domain page if not done yet.
				Log::F($favicon,'not found in page try /');
				if ($instance->CallAgain($favicon,'/') == $instance::FLAG_DATA_FOUND)
				{
					return $instance::FLAG_DATA_FOUND;
				}
				else
				{
					$favicon->SetSource('sources_root');
					Log::F($favicon,'not found in page try /favicon.ico');
					return $instance->CallAgain($favicon,'/favicon.ico');
				}
			}
			return false;
		}

		/* ------------------------------------------------------------ */


		/**
		 * This function return an icon inside an array based on a uri request
		 * @param array $r_arr
		 * @returns array $r_arr
		 */
		function GetIconFromUri($favicon, $state='')
		{
			Log::F($favicon,'!');
			$instance = self::getInstance();
			if ($favicon->LastUriAddedToUriArr() == '')
			{
				return $instance::FLAG_NOK;
			}
			if ( $favicon->CountUriArr() > $instance::MAX_REDIRECTS)
			{
				Log::F($favicon,'TOO MANY REDIRECTS');
				return $instance::FLAG_TOO_MANY_COUNTS;
			}

			/* --------------------------------------------------------------
			 * HTTP
			 * --------------------------------------------------------------
			 */

			/* 4.1 HEAD : first try only the head to see if we get a redirect or error */

	//
	// Based on all the cases of head that give 200 and gets that give
	// 303 or 'method not allowed' where a get is needed ... I wonder if I
	// need to do HEAD requests first at all (@todo to investigate)
	// also: the majority of cases will not need 2 requests but 1 request is
	// enough but I dont know if it will catch all 301/302 with only get
	// requests... lots of weird things out there.
	// but even sites like http://www.mymobiler.com/ who produce 301 while
	// a regular request just returns the body so... I'm going to ditch
	// the HEAD requests.

//			$response = $instance->HttpRequest($favicon, 'HEAD');
//			Log::F($favicon,'HEAD GET ' . $favicon->LastUriAddedToUriArr());
//			if ( is_wp_error( $response ))
//			{
//				Log::F($favicon,'HEAD ERROR ' . $response->get_error_message());
//				return $instance::FLAG_NOK;
//			}

			/* HEAD : Returns 200 : do full request (do not log the HEAD
			 * request to keep unique uri
			 * HEAD : Returns 404 : do full request (since even most 404
			 * pages give many nice pages to scan for favicons)
			 * HEAD : Returns 405 : do full request (since in most cases the head
			 * request is not supported)
			 * */
//			elseif (	$response['response']['code'] == 200
//					||  $response['response']['code'] == 404
//					||	$response['response']['code'] == 405
//					)
//			{
//				if (isset($response['headers']['content-type'])) {
//					Log::F($favicon,'HEAD 4.3 200/404/405 MIMETYPE : '
//					. $response['headers']['content-type']);
//				}
//				else
//				{
//					Log::F($favicon,'HEAD 4.3 200/404/405 No mimetype ');
//				}

				$response = $instance->HttpRequest($favicon,'FULL');
				Log::F($favicon,'GET ' . $favicon->LastUriAddedToUriArr());
				/* GET : ERROR */
				if ( is_wp_error( $response ))
				{
					Log::F($favicon,'GET ERROR : '. $favicon->LastUriAddedToUriArr() . ',' . $response->get_error_message());
					return $instance::FLAG_NOK;
				}
				/* GET : 200 */
				elseif ($response['response']['code'] == 200)
				{
					if ($data = $response['body'])
					{
						$favicon->SetData($data);
						if ($instance->CheckMimeType($response['headers']['content-type']))
						{
							Log::F($favicon,'GOT ICON : ' . $favicon->LastUriAddedToUriArr());
							return $instance::FLAG_DATA_FOUND;
						}
						else
						{
							Log::F($favicon,'Try find icon in PAGE : ' . $favicon->LastUriAddedToUriArr());
							return $instance->GetHrefStringFromHTMLPage($favicon);
						}
					}
					else
					{
						Log::F($favicon,'GET 200 but no BODY : ' . $favicon->LastUriAddedToUriArr());
						return $instance::FLAG_NOK;
					}
				}
				/* GET 301/302
				 * Solved this one due to https://core.trac.wordpress.org/ticket/16855
				 * (Hakre for the fix, TheDeadMedic, dd32) so by the time this comes out...
				 * it should be in a WP Release.
				 */
				elseif (in_array($response['response']['code'], array('301', '302')))
				{
					Log::F($favicon,'GET 301/302 - moved - try redirect');
					if (is_array($response['headers'])) {
						if (array_key_exists('location',$response['headers']))
						{
							return $instance->CallAgain($favicon,$response['headers']['location']);
						}
						else
						{
							Log::F($favicon,'NO REDIRECT URL IN REDIRECT REQUEST');
							return $instance::FLAG_NOK;
						}
					}
					else
					{
						Log::F($favicon,'NO REDIRECT HEADERS IN REDIRECT REQUEST');
						return $instance::FLAG_NOK;
					}
				}
				/* GET : 403 - forbidden - try root */
				elseif (in_array($response['response']['code'], array('403')))
				{
					Log::F($favicon,'HEAD 403 - Forbidden - try root');
					return $instance->CallAgain($favicon,'/');
				}
				/* 4.2.2 GET : 404 (could contain useful stuff) */
				elseif ($response['response']['code'] == 404)
				{
					$data = $response['body'];
					if ($data)
					{
						/* there might be an image link in the HTML */
						if ( strpos(strtolower("-".$data),strtolower('<html')) &&
							 strpos(strtolower("-".$data),strtolower('<body')) )
						{
							/* its HTML */
							Log::F($favicon,'GET 404 : Try find icon in PAGE : '
							. $favicon->LastUriAddedToUriArr());
							$favicon->SetData($data);
							return $instance->GetHrefStringFromHTMLPage($favicon);
						}
						else
						{

							Log::F($favicon,'FULL GET 404 : body but no html try root '
							. $favicon->LastUriAddedToUriArr());
							return $instance->CallAgain($favicon,'/');
						}
					}
					else
					{
						Log::F($favicon,'FULL GET 404 : no body try root' . $favicon->LastUriAddedToUriArr());
						return $instance->CallAgain($favicon,'/');
					}
				}
				/* GET : 405 - Method not allowed*/
				elseif (in_array($response['response']['code'], array('405')))
				{
					Log::F($favicon,'GET 405 - Method not allowed - try root favicon last shot');
					return $instance->CallAgain($favicon,'/favicon.ico');
				}
				/* GET: other than 200 / 404 : log it to analyze assume no redirect */
//				else
//				{
//					/* first check the mime type returned : page = do again */
//					Log::F($favicon,'GET NON 200/404/405/301/302 : '
//					. $response['response']['code']);
//					return $instance::FLAG_NOK;
//				}
				/* GET : Other */
				else
				{
					Log::F($favicon,'HEAD NNN - OTHER RETURN CODE:' , in_array($response['response']['code']));
					return $instance::FLAG_NOK;
				}
			}
			//@ todo what is http://www.ibmpressbooks.com/series/ doing?
/*			elseif (in_array($response['response']['code'], array('301', '302')))
			{
				Log::F($favicon,'HEAD 301/302 - moved - try redirect');
				if (is_array($response['headers'])) {
					if (array_key_exists('location',$response['headers']))
					{
						return $instance->CallAgain($favicon,$response['headers']['location']);
					}
					else
					{
						Log::F($favicon,'NO REDIRECT URL IN REDIRECT REQUEST');
						return $instance::FLAG_NOK;
					}
				}
				else
				{
					Log::F($favicon,'NO REDIRECT HEADERS IN REDIRECT REQUEST');
					return $instance::FLAG_NOK;
				}
			}
			elseif (in_array($response['response']['code'], array('403')))
			{
				Log::F($favicon,'HEAD 403 - Forbidden - try root');
				return $instance->CallAgain($favicon,'/');
			}
			elseif (in_array($response['response']['code'], array('405')))
			{
				Log::F($favicon,'HEAD 405 - Method not allowed - mostly HEAD requests are not allowed');
				return $instance->CallAgain($favicon,'/favicon.ico');
			}
			else
			{
				Log::F($favicon,'HEAD NNN - OTHER RETURN CODE');
				return $instance::FLAG_NOK;
			}
		}*/

		/* ------------------------------------------------------------ */

		/**
		 *
		 * Enter description here ...
		 * @param Favicon $favicon the current favicon object
		 * @param string $dataUri the data uri to get
		 * @param bool $icon_or_page if it is an icon or page request for logging
		 * @param string $source the source module calling this
		 * @param bool $default is used as default icon or not
		 * @todo icon or page parameter is not needed
		 */
		function GetFavicon($favicon, $source, $default)
		{
			$instance = self::getInstance();
			$favicon->AddUriToUriArr($favicon->GetFaviconUri());
			if (!$favicon->FoundStatus() && ($instance::FLAG_DATA_FOUND ==
				$instance->GetIconFromUri($favicon, '')))
			{
				$favicon->Found();
				// @todo next 3 line can go in calling functions themselves
				$favicon->SetDefault($default);
				$favicon->SetSource($source);
				$favicon->SetFaviconUri($favicon->LastUriAddedToUriArr());
			}
			else
			{
				$favicon->EmptyUriArr();
			}
		}

		/* ------------------------------------------------------------ */

		/**
		 *
		 * Enter description here ...
		 * @param $favicon
		 * @param $msg1
		 * @param $msg2
		 * @param $uri2
		 */
		function CallAgain($favicon,$uri2)
		{
			$instance = self::getInstance();
			if ($new_uri
					= $instance->BuildRedirectedUri($favicon
						,$instance->getFullFaviconPath($favicon->LastUriAddedToUriArr(),$uri2)))
			{
				$favicon->AddUriToUriArr($new_uri);
				return $instance->GetIconFromUri($favicon, '');
			}
			else
			{
				Log::F($favicon,'Already Tried or Invalid');
				return $instance::FLAG_NOK;
			}
		}

		/**
		 *
		 * Enter description here ...
		 * @param $favicon
		 * @param $new_uri
		 */
		function WentThereBefore($favicon,$new_uri)
		{
			$went_there_before = false;
			foreach($favicon->GetUriArr() as $uri)
			{
				if ($uri == $new_uri)
				{
					$went_there_before = true;
				}
			}
			return $went_there_before;
		}

		/**
		 * Builds a new uri to go to after redirect
		 * @param string $old_uri
		 * @param string $new_uri
		 * @return string $new_uri
		 * @todo check for condition that old url is the same as the previous one
		 */
		function BuildRedirectedUri($favicon,$new_uri)
		{
			$old_uri = $favicon->LastUriAddedToUriArr();
			if (in_array(parse_url($new_uri,PHP_URL_SCHEME), array('http','https','file'))) {
				if ($this->WentThereBefore($favicon,$new_uri))
				{
					return false;
				}
				else
				{
					return $new_uri;
				}
			}
			$old_url_scheme = parse_url($old_uri,PHP_URL_SCHEME) . '://';
			$old_url_host = parse_url($old_uri, PHP_URL_HOST);
			$old_url_port = parse_url($old_uri, PHP_URL_PORT);
			if ($old_url_port)
			{
				$old_url_port = ':' . $old_url_port;
			}
			$old_uri = $old_url_scheme . $old_url_host . $old_url_port;

			if (substr($new_uri,1) == '/')
			{

				$new_uri = $old_uri . $new_uri;
			}
			else
			{
				$new_uri = $old_uri . '/' . $new_uri;
			}
			/* check that we did not go there before */
			$went_there_before = false;
			foreach($favicon->GetUriArr() as $uri)
			{
				if ($uri == $new_uri)
				{
					$went_there_before = true;
				}
			}
			if ($this->WentThereBefore($favicon,$new_uri))
			{
				return false;
			}
			else
			{
				return $new_uri;
			}
		}

	   /**
		 * Checks if a returned header content type is image
		 * @param $content_type
		 * @return bool $return_type
		 */
		function CheckMimeType($content_type)
		{
			$return_type = false;
			if ($content_type)
			{
				if (strpos($content_type,'text') !== false)
				{
					$return_type = false;
				}
				elseif(strpos($content_type,'image') !== false)
				{
					$return_type = true;
				}
			}
			return $return_type;
		}

		/**
		 * check if a uri host is an IP address
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
	 	 * @copyright GPL 2
		 * @param $ip
		 */
		function valid_ip($ip) {
			return preg_match("/^([1-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])" .
				"(\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$/", $ip);
		}


		/**
		 * deliver an image url from a page string
		 * @since 0.4.0
		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
		 * @param object $xpath
		 * @param string $XpathExpr
		 * @param string $url
		 */
		function XpathHrefCheck($xpath, /*string*/ $XpathExpr)
		{
			$elements = $xpath->query($XpathExpr);
			if (!is_null($elements) && $elements!=false)
			{
				foreach ($elements as $element)
				{
					if ($element->getAttribute('href'))
					{
						$href = $element->getAttribute('href');
						return $href;
					}
				}
			}
			return false;
		}

		/**
 		 * Gets the full image path for a found image in a link href
 		 * @since 0.4.0
 		 * @author Edward de Leau <e@leau.net>, {@link http://wp.leau.co}
 	 	 * @param $url
 		 * @param $image
 		 */
		function getFullFaviconPath($url, $image='favicon.ico', $xpath='')
		{
			$instance = self::getInstance();

			/* image paths with scheme are easy: */

			if ($url_scheme = parse_url($image,PHP_URL_SCHEME)) {
				Log::M(' [FROMPAGE] full path: ' . $image);
				return $image;
			}
			/* especially for google: <link rel="shortcut icon" href="//www.google.com/favicon.ico"> */
			if (substr($image,0,2)=='//')
			{
				Log::M(' [FROMPAGE] full path: ' . 'http:' . $image);
				return 'http:' . $image;
			}
			$base_href = false;
			/* for relative paths we need to check first if the base href is set */
			if ($xpath)
			{
				$XpathExpr = "//base";
				$base_href = $instance->XpathHrefCheck($xpath, $XpathExpr);
				// now some sites such as www.shoutmouth.com have:
				// http://www.shoutmouth.com/index.php...
			}
			if ($base_href)
			{
				/* is the base href relative/ */
				if (!parse_url($base_href,PHP_URL_SCHEME))
				{
					$base_href_scheme_temp = parse_url($url,PHP_URL_SCHEME) . "://";
					$base_href__host_temp = parse_url($url, PHP_URL_HOST);
					$base_href__port_temp = parse_url($url, PHP_URL_PORT);
					if ($base_href__port_temp) {
						$base_href__port_temp = ':' . $base_href__port_temp;
					}
					$base_href_temp=$base_href_scheme_temp . $base_href__host_temp .
						$base_href__port_temp . '/';
					$base_href = $base_href_temp . $base_href;
				}
				/* is it the file protocol? */
				if ($base_href=='file')
				{
					if (!substr($base_href,-2)=='\\\\')
					{
						$base_href = $base_href . '\\\\';
					}
				}
				else
				{
					if (!substr($base_href,-1)=='/')
					{
						$base_href = $base_href . '/';
					}
				}
			}
			else
			{
					$base_href_scheme = parse_url($url,PHP_URL_SCHEME) . "://";
					$base_href__host = parse_url($url, PHP_URL_HOST);
					$base_href__port = parse_url($url, PHP_URL_PORT);
					if ($base_href__port) {
						$base_href__port= ':' . $base_href__port;
					}
					$base_href=$base_href_scheme . $base_href__host . $base_href__port . '/';
			}
			if (substr($image,0,1)=='/')
			{
				$image = substr($image,1);
			}
			$image_string = $base_href . $image;

			Log::M('TRYING image string:' . $image_string);

			return $image_string;
		}

		/**
		 * @param $uri
		 * @param $timeout
		 * @param $requesttype
		 */
		function HttpRequest($favicon, $requesttype)
		{
			if( !ini_get('safe_mode') ){
				ini_set('max_execution_time',1000);
				set_time_limit(1000);
			}
			$response='';
			$version = Config::GetPluginVersion();
			$http_args_get = array (
				'method' => 'GET',
				'timeout' => 300,
				'redirection' => 0,
				'httpversion' => '1.0',
				'user-agent' => "WordPress/WP-Favicons/$version; " . get_bloginfo( 'url'),
				'blocking' => true,
				'headers' => array(),
				'cookies' => array(),
				'body' => null,
				'compress' => false,
				'decompress' => true,
				'sslverify' => false,
			    'followlocation' => false
			);
			$http_args_head = array (
				'method' => 'HEAD',
				'timeout' => 300,
				'redirection' => 0,
				'httpversion' => '1.0',
				'user-agent' => "WordPress/WP-Favicons/$version; " . get_bloginfo( 'url'),
				'blocking' => true,
				'headers' => array(),
				'cookies' => array(),
				'body' => null,
				'compress' => false,
				'decompress' => true,
				'sslverify' => false,
			    'followlocation' => false
			);

			//$timeout = 300;
			//$instance = self::getInstance();
			//add_filter( 'http_request_redirection_count', array($instance,'SetDirectionCount'));
			//add_filter( 'http_headers_useragent', array($instance,'SetHeaderUserAgent'));
			// underneath the same as pre_http_request
			$response = apply_filters( Config::GetPluginSlug() . 'pre_http', $favicon, $response);
			if (!$response)
			{
				if ($requesttype=='FULL')
				{
					$response = wp_remote_get($favicon->LastUriAddedToUriArr(), $http_args_get);
					do_action(Config::GetPluginSlug() . 'post_http_full', $favicon, $response);
				}
				elseif ($requesttype = 'HEAD')
				{
					$response = wp_remote_head($favicon->LastUriAddedToUriArr(), $http_args_head);
					do_action(Config::GetPluginSlug() . 'post_http_head', $favicon, $response);
				}
			}
			return $response;
		}
	}
}
