<?php
/**
 * Contains Classes for the Href operations.
 *
 * @package WP-Favicons
 * @since 0.030
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */
namespace leau\co\wp_favicons;
/**
 * abstract class for WordPress Href operations
 *
 * The following abstract class provides methods to work with HTML Hrefs
 * such as putting strings in front or after a href
 *
 * @package WP-Favicons
 * @since 0.1
 * @author Edward de Leau <e@leau.net>
 * @copyright Edward de Leau, http://edward.de.leau.net
 */
if (!class_exists("EdlHtmlHref")) {

	abstract class EdlHtmlHref {

		const HTML_REF_REGEX2 = '/<a(.*?)href=[\'"](.*?)[\'"](.*?)>(.*?)<\\/a>/i';
		public $mStrNotAllowedExtensions = '';
		abstract public function SpecificHrefBefore($strHtmlUri);

		/*
		 * function: set not allowed extensions
		 */
		public function SetNotAllowedExtension( /*string*/ $strNotAllowedExtensions)
		{
			if(!is_string($strNotAllowedExtensions)){
				throw new Exception('Invalid parameter type!');
			}
			$this->mStrNotAllowedExtensions = $strNotAllowedExtensions;
		}



		/**
		 * Validates the Href
		 * Pretty important to determine what we want to support,
		 * @param unknown_type $arrUrlMatches
		 */
		private function ValidateHTMLHref($arrUrlMatches)
		{
			// let's start to suppose it is valid (be positive :))
			$bool_href_valid=true;

			// give the matches some decent names again
			$str_original_uri_attributes_before = $arrUrlMatches[1];
			$str_original_uri_uri = $arrUrlMatches[2];
			$str_original_uri_attributes_after = $arrUrlMatches[3];
			$str_original_uri_display_text = trim($arrUrlMatches[4]);

			/*
			 * (!) protocols supported:
			 * - http://
			 * - https://
			 * - file://
			 * no refinement on placing other content within it at this stage
			 * STRICT! In next release some DOM within these tags support
			 */
			if (!((substr($str_original_uri_uri,0,7) == 'http://' ) ||
				  (substr($str_original_uri_uri,0,8) == 'https://') ||
				  (substr($str_original_uri_uri,0,7) == 'file://' ) )
			    )
			{
				return false;
			}

			/* allow no tags in href strict */
			if (strstr(trim($str_original_uri_display_text), '<'))
			{
				return false;
			}


			/* user defined extensions will also not be replaced */
			$str_original_uri_type_extension
				= EdlUtils::GetUriExtension($str_original_uri_uri, '.', 1, 1);
			if ($str_original_uri_type_extension != false)
			{
				if (stristr($this->mStrNotAllowedExtensions,
					$str_original_uri_type_extension) != false)
				{
					$bool_href_valid=false;
				}
			}

			return $bool_href_valid;
		}

	   /**
		* Called for each Href found by ReplaceAll
		* @param array $arrUrlMatches
		* @return string
		*/
		private function AddExtraHtmlBeforeOneHref($arrUrlMatches)
		{
			$content = '<a' . $arrUrlMatches[1] . 'href="' . $arrUrlMatches[2]
				. '"' . $arrUrlMatches[3] .'>' . $arrUrlMatches[4] . '</a>';

			/* if the href is valid add things otherwise dont */
			//
			// @todo we should place uri's here on a URI queue so that
			// people can distribute the load over multiple servers
			//
			if ($this->ValidateHTMLHref($arrUrlMatches)) {
				return $this->SpecificHrefBefore($arrUrlMatches[2]) .
					$content;
			} else {
				return $content;
			}
		}

		/**
		 * this public method is the entry point for classes
		 * to replace hrefs in body text
		 *
		 * @todo main problem is that this one is being called time and time
		 * again to replace the content. the filter is just "on" so if we
		 * would have somewhat more metadata on where we were we could also
		 * cache these calls and only perform them occasionally that would
		 * be nice since if a piece of content does not have any hrefs
		 * to replace we are checking this time and time again what we would
		 * need is some kind of unique id over all possible filters used
		 * in wordpress that can be calculated e.g. sometimes the commentid,
		 * sometimes the postid OR.... mmmmmmmm .... maybe I can md5 the content
		 * and then store that one as unique ID and then check for that MD5
		 * to prevent any more processing directly. So this is the cost of one
		 * database call versus the cost of calling the functions in this page.
		 * then again that would be lots of database calls for all filters then
		 * again we could limit that checks on content to e.g. once per 5 minutes
		 *
		 * @todo in the refactoring of this "presentation module"
		 * we will make a class class UriMetadata with gets a factory as input
		 * this then pits the metadataline together
		 *
		 * the factory object is then responsible for the interface such as
		 * now specificHrefBefore which is ow favicontcontent
		 *
		 * @param string $content
		 * @return string $changedContent
		 */
		public function ReplaceAll($content)
		{
			$r_content = $content;
			if (is_string($content))
			{
				$r_content = preg_replace_callback(self::HTML_REF_REGEX2,
					array($this,'AddExtraHtmlBeforeOneHref'), $content);
			}
			// $md5_content = md5($content);
			return $r_content;
		}
	}
}